/* vi:set wm=0 ai tabstop=4 shiftwidth=4: */

/*
 * sctl - Bearcat scanner control program
 * Copyright (C) 2001 Ian Johnston <reaper@muppetlabs.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "human_commands.h"
#include "commands.h"
#include "config.h"
#include "usage.h"
#include "utils.h"
#include "serial.h"
#include "main.h"

/***********************************************************************
 * RF attenuation command
 ***********************************************************************/
void human_do_att(int fd, int argc, char **args)
{
	if (args[0] == NULL)
		att(fd, -1);
	else if (strcmp(args[0], "on") == 0)
		att(fd, 1);
	else if (strcmp(args[0], "off") == 0)
		att(fd, 0);
	else
		usage(C_ATT);
}


/***********************************************************************
 * Data skip command
 ***********************************************************************/
void human_do_data(int fd, int argc, char **args)
{
	if (args[0] == NULL)
		data(fd, -1);
	else if (strcmp(args[0], "on") == 0)
		data(fd, 1);
	else if (strcmp(args[0], "off") == 0)
		data(fd, 0);
	else
		usage(C_DATA);
}


/***********************************************************************
 * Delay command
 ***********************************************************************/
void human_do_delay(int fd, int argc, char **args)
{
	if (args[0] == NULL)
		delay(fd, -1);
	else if (strcmp(args[0], "on") == 0)
		delay(fd, 1);
	else if (strcmp(args[0], "off") == 0)
		delay(fd, 0);
	else
		usage(C_DELAY);
}


/***********************************************************************
 * Report talkgroup IDs
 ***********************************************************************/
void human_do_id_report(int fd, int argc, char **args)
{
	id_report(fd);
}

/***********************************************************************
 * Dump command
 ***********************************************************************/
void human_do_dump(int fd, int argc, char **args)
{
	char bank;
	FILE *outfile;
	char errorstr[100];

	if (args[0] == NULL)
	{
		complain("Must include a bank id.");
		usage(C_DUMP);
		return;
	}
	
	sscanf(args[0], "%c", &bank);
	bank = toupper(bank);
	if (bank >= 'A' && bank <= 'J') {
		if (args[1] == NULL)
			dump(fd, bank, stdout);
		else
		{
			if ((outfile = fopen(args[1], "w")) != NULL)
				dump(fd, bank, outfile);
			else
			{
				snprintf(errorstr, 100, "dump to \"%s\" failed", args[1]);
				perror(errorstr);
				return;
			}
		}
	}
	else
	{
		complain("Must include a bank id between A and J.");
		usage(C_DUMP);
		return;
	}
}


/***********************************************************************
 * Light command
 ***********************************************************************/
void human_do_light(int fd, int argc, char **args)
{
	if (args[0] == NULL)
		light(fd, -1);
	else if (strcmp(args[0], "on") == 0)
		light(fd, 1);
	else if (strcmp(args[0], "off") == 0)
		light(fd, 0);
	else if (strcmp(args[0], "auto") == 0)
		light(fd, 3);
	else if (strcmp(args[0], "noauto") == 0)
		light(fd, 2);
	else
		usage(C_LIGHT);
}


/***********************************************************************
 * Load command
 ***********************************************************************/
void human_do_load(int fd, int argc, char **args)
{
	char bank;
	FILE *infile;
	char errorstr[100];

	if (args[0] == NULL)
	{
		complain("Must include a bank id.");
		usage(C_LOAD);
		return;
	}
	
	sscanf(args[0], "%c", &bank);
	bank = toupper(bank);
	if (bank >= 'A' && bank <= 'J') {
		if (args[1] == NULL)
			load(fd, bank, stdin);
		else
		{
			if ((infile = fopen(args[1], "r")) != NULL)
				load(fd, bank, infile);
			else
			{
				snprintf(errorstr, 100, "load from \"%s\" failed", args[1]);
				perror(errorstr);
				return;
			}
		}
	}
	else
	{
		complain("Must include a bank id between A and J.");
		usage(C_LOAD);
		return;
	}
}


/***********************************************************************
 * Lockout command
 ***********************************************************************/
void human_do_lockout(int fd, int argc, char **args)
{
	int i, cmd, force;

	force = 0;
	cmd = -1;

	if (args[0] == NULL)
		cmd = -1;
	else
	{
		for (i=0; i<argc; i++)
		{
			if (strcmp(args[i], "on") == 0)
			{
				cmd = 1;
				continue;
			}

			if (strcmp(args[i], "off") == 0)
			{
				cmd = 0;
				continue;
			}

			if (strcmp(args[i], "-f") == 0)  /* undocumented */
			{
				force = 1;
				continue;
			}

			usage(C_LOCKOUT);
		}
	}

	lockout(fd, cmd, force);
}


/***********************************************************************
 * Mem command
 ***********************************************************************/
void human_do_mem(int fd, int argc, char **args)
{
	int mem_no, i, force, mult, mem_upper, mem_lower, set;
	double freq;
	char errstring[100];

	mem_upper = 0;	/* upper limit user specified for memory range */
	mem_lower = 0;	/* lower limit user specified for memory range */
	mult = 0;		/* whether they have specified multiple memory locations */
	force = 0;		/* whether they want to force to manual mode */
	set = 1;		/* if they want to change to the memory location */
	mem_no = -1;	/* which memory location */
	freq = -1;		/* set frequency to this mhz */

	for (i=0; i<argc; i++)
	{
		if (strcmp(args[i], "-f") == 0)
		{
			force = 1;
			continue;
		}
		
		if (strcmp(args[i], "-q") == 0)
		{
			set = 0;
			continue;
		}
		
		if (strcmp(args[i], "-s") == 0)
		{
			if (sscanf(args[++i], "%lf", &freq))
				continue;
		}

		if (strcmp(args[i], "-c") == 0)
		{
			sscanf(args[++i], "%d-%d", &mem_lower, &mem_upper);
			mult = 1;
			set = 0;
			continue;
		}

		if (sscanf(args[i], "%d", &mem_no))
		{
			if (mem_no > MAXMEM)
			{
				snprintf(errstring, 100, "Must select a memory between 1 and %d.",
					MAXMEM);
				complain(errstring);
				usage(C_MEM);
			}
			else if (mem_no < 1)
			{
				snprintf(errstring, 100, "Must select a memory between 1 and %d.",
					MAXMEM);
				complain(errstring);
				usage(C_MEM);
			}
			else
			{
				continue;
			}
		}

		if (strcmp(args[i], "all") == 0)
		{
			mem_lower = 1;
			mem_upper = MAXMEM;
			mult = 1;
			continue;
		}

		/* if we get this far the user has included funky options */
		usage(C_MEM);
	}

	if (mult == 0)
	{
		if (set == 0 && mem_no == -1)
		{
			complain("Query mode requires a channel number (give no args to check current\n\tmemory).");
			usage(C_MEM);
		}

		mem(fd, mem_no, freq, force, set);
	}
	else
	{
		if (mem_no != -1 || freq != -1)
		{
			complain("Cannot set frequency or specify a single freq with multiple option.");
			usage(C_MEM);
		}

		for (i=mem_lower; i<=mem_upper; i++)
		{
			mem(fd, i, -1, force, 0);
		}
	}
}


/***********************************************************************
 * Mute command
 ***********************************************************************/
void human_do_mute(int fd, int argc, char **args)
{
	int mute_val;

	if (args[0] == NULL)
		mute_val = -1;
	else if (strncmp(args[0], "o", 1) == 0) /* aka "off" */
		mute_val = 0;
	else if (strncmp(args[0], "a", 1) == 0) /* aka "all" */
		mute_val = 1;
	else if (strncmp(args[0], "n", 1) == 0) /* aka "normal" */
		mute_val = 2;
	else
		usage(C_MUTE);

	mute(fd, mute_val);
}


/***********************************************************************
 * Scan command
 ***********************************************************************/
void human_do_scan(int fd, int argc, char **args)
{
	char banks[11], temp_bank;
	int i, do_trunk, bank_i;

	bank_i = 0;
	do_trunk = 0;

	for (i=0; i<argc; i++)
	{
		if (strcmp(args[i], "-t") == 0)
		{
			do_trunk = 1;
			continue;
		}

		if (sscanf(args[i], "%c", &temp_bank))
		{
			if (('a' - 1) < temp_bank && temp_bank < 'k') 
			{
				temp_bank = toupper(temp_bank);
				banks[bank_i++] = temp_bank;
				continue;
			}
			else if (('A' - 1) < temp_bank && temp_bank < 'K')
			{
				banks[bank_i++] = temp_bank;
				continue;
			}
			else
			{
				complain("Banks must be between A and J.");
				return;
			}
		}
	}

	/* "terminate" the "string" so we can find the end */
	banks[bank_i] = '\0';

	if (!banks[0])
	{
		complain("Must include at least one valid bank id.");
		usage(C_SCAN);
		return;
	}

	if (do_trunk)
	{
		scan_trunk(fd, banks);
	}
	else
	{
		scan(fd, banks);
	}

}


/***********************************************************************
 * Scan report command
 ***********************************************************************/
void human_do_scan_report(int fd, int argc, char **args)
{
	char banks[11], temp_bank;
	int i, bank_i;

	bank_i = 0;

	for (i=0; i<argc; i++)
	{
		if (sscanf(args[i], "%c", &temp_bank))
		{
			if (('a' - 1) < temp_bank && temp_bank < 'k') 
			{
				temp_bank = toupper(temp_bank);
				banks[bank_i++] = temp_bank;
				continue;
			}
			else if (('A' - 1) < temp_bank && temp_bank < 'K')
			{
				banks[bank_i++] = temp_bank;
				continue;
			}
			else
			{
				complain("Banks must be between A and J.");
				return;
			}
		}
	}

	/* "terminate" the "string" so we can find the end */
	banks[bank_i] = '\0';

	if (!banks[0])
	{
		complain("Must include at least one valid bank id.");
		usage(C_SCAN_REPORT);
		return;
	}

	scan_report(fd, banks);

}


/***********************************************************************
 * Search command
 ***********************************************************************/
void human_do_search(int fd, int argc, char **args)
{
	double freq1 = 0, freq2 = 0;

	if (args[0] == NULL)
	{
		search(fd);
	}
	else if (args[1] == NULL)
	{
		complain("Please supply two frequencies (an upper and lower limit).");
		usage(C_SEARCH);
	}
	else
	{
		sscanf(args[0], "%lf", &freq1);
		sscanf(args[1], "%lf", &freq2);
		if (limit(fd, 0, freq1, freq2) == 1)
			search(fd);
	}
}


/***********************************************************************
 * status command
 ***********************************************************************/
void human_do_status(int fd, int argc, char **args)
{
	int status, i_temp;
	char c_temp[100], c_temp2[100], answer[100];
	double d_temp;

	bzero(c_temp, 100);
	bzero(c_temp2, 100);
	bzero(answer, 100);

	status = get_mode(fd);
	printf("Current status: %s\n", mode_desc[status]);
	switch(status)
	{
		case M_MANUAL:
			mem(fd, -1, -1, 0, 1);

			if (scannermodel == BC895XLT || scannermodel == BC780XLT)
			{
				bzero(answer,100);
				bzero(c_temp, 100);
				bzero(c_temp2, 100);
				do_command(fd, "SG", answer);
				sscanf(answer, "S%3s F%8s", c_temp, c_temp2);
				d_temp = bc2human_freq(c_temp2);
				printf("Listening to %.4f MHz, Signal strength %s  ", d_temp,
					c_temp);

				if (scannermodel == BC895XLT)
				{
					print_s_meter(c_temp);
					putchar('\n');
				}
				else
				{
					putchar('\n');
				}
			}
			break;

		case M_CHAN_SCAN:
			do_command(fd, "SB", answer);
			sscanf(answer, "SB %s", c_temp);
			printf("Scanning banks %s.\n", c_temp);
			
				bzero(answer,100);
				bzero(c_temp, 100);
				bzero(c_temp2, 100);
			if (scannermodel == BC895XLT || scannermodel == BC780XLT)
			{
				do_command(fd, "SG", answer);
				sscanf(answer, "S%3s F%8s", c_temp, c_temp2);
				d_temp = bc2human_freq(c_temp2);
				printf("Listening to %.4f MHz, Signal strength %s  ", d_temp, 
					c_temp);

				if (scannermodel == BC895XLT)
				{
					print_s_meter(c_temp);
					putchar('\n');
				}
				else
				{
					putchar('\n');
				}
			}
			else
			{
				do_command(fd, "WI", answer);
				sscanf(answer, "W%3s F%8s", c_temp, c_temp2);
				d_temp = bc2human_freq(c_temp2);
				/* c_temp is "window voltage", whatever the hell that is */
				printf("Listening to %.4f MHz.\n", d_temp);
			}
			break;

		case M_LIMIT_SRCH:
		case M_LIMIT_SRCH_HOLD:
			limit(fd, 1, 0, 0);
			break;

		case  M_M_ID_SRCH:
		case  M_M_ID_SRCH_HOLD:
		case  M_M_ID_SCAN:
		case  M_M_ID_MANUAL:
			do_command(fd, "IC", answer);
			if (sscanf(answer, "IC %2s %6d", c_temp, &i_temp) == 2)
			{
				printf("Listening to ID memory %s, talkgroup ID %d.\n", c_temp,
					i_temp);
			}
			else if (sscanf(answer, "IC %2s ------", c_temp) == 1)
			{
				printf("Listening to ID memory %s, talkgroup ID unset.\n",
					c_temp);
			}
			if (scannermodel == BC895XLT || scannermodel == BC780XLT)
			{
				bzero(answer, 100);
				bzero(c_temp, 100);
				bzero(c_temp2, 100);
				do_command(fd, "SG", answer);
				sscanf(answer, "S%3s F%8s", c_temp, c_temp2);
				d_temp = bc2human_freq(c_temp2);
				printf("Listening to %.4f MHz, Signal strength %s  ", d_temp,
					c_temp);

				if (scannermodel == BC895XLT)
				{
					print_s_meter(c_temp);
					putchar('\n');
				}
				else
				{
					putchar('\n');
				}
			}
			break;

		default:
			printf("Additional information not available.\n");
			break;
	}

}


/***********************************************************************
 * Talkgroup command
 ***********************************************************************/
void human_do_talkgroup(int fd, int argc, char **args)
{
	/* sctl talk J C7 -s 1774 */
	int i, loc, groupid, all;
	char bank, scanbank;
	char memloc[3];

	groupid = -1;
	loc = -1;
	all = 0;
	bank = '\0';
	scanbank = 'Z';
	bzero(memloc, 3);

	for (i=0; i<argc; i++)
	{
		if (strcmp(args[i], "-s") == 0)
		{
			if (i+1 <= argc && sscanf(args[++i], "%d", &groupid))
			{
				continue;
			}
			else
			{
				complain("-s not followed by a talkgroup id.");
				usage(C_TALKGROUP);
			}
		}

		if (sscanf(args[i], "%c%d", &bank, &loc) == 2)
		{
			if ('a' <= bank && bank <= 'e') 
			{
				bank = toupper(bank);
				snprintf(memloc, 3, "%c%d", bank, loc);
				continue;
			}
			else if ('A' <= bank && bank <= 'E')
			{
				snprintf(memloc, 3, "%c%d", bank, loc);
				continue;
			}
			else
			{
				complain("Memory banks must be between A and E.");
				usage(C_TALKGROUP);
			}
		}

		if (strcmp(args[i], "all") == 0)
		{
			all = 1;
			continue;
		}

		if (sscanf(args[i], "%c ", &scanbank) == 1)
		{
			if ('a' <= scanbank && scanbank <= 'j') 
			{
				scanbank = toupper(scanbank);
				continue;
			}
			else if ('A' <= scanbank && scanbank <= 'J')
			{
				continue;
			}
			else
			{
				complain("Scan banks must be between A and J.");
				usage(C_TALKGROUP);
			}
		}

		usage(C_TALKGROUP);
	}

	if (all == 0)
	{
		talkgroup(fd, memloc, groupid, scanbank);
	}
	else
	{
		talkgroup_all(fd, scanbank);
	}
}


/***********************************************************************
 * Version command
 ***********************************************************************/
void human_do_version(int fd, int argc, char **args)
{
	version(fd);
}


/***********************************************************************
 * get help
 ***********************************************************************/
void human_help(int fd, int argc, char **args)
{
	if (args[0] == NULL)
		usage(C_NONE);

	if (strncmp(args[0], "a", 1) == 0)
		usage(C_ATT);
	else if (strncmp(args[0], "da", 2) == 0)
		usage(C_DATA);
	else if (strncmp(args[0], "de", 2) == 0)
		usage(C_DELAY);
	else if (strncmp(args[0], "du", 2) == 0)
		usage(C_DUMP);
	else if (strncmp(args[0], "h", 1) == 0)
		usage(C_HELP);
	else if (strncmp(args[0], "i", 1) == 0)
		usage(C_ID_REPORT);
	else if (strncmp(args[0], "lig", 3) == 0)
		usage(C_LIGHT);
	else if (strncmp(args[0], "loa", 3) == 0)
		usage(C_LOAD);
	else if (strncmp(args[0], "loc", 3) == 0)
		usage(C_LOCKOUT);
	else if (strncmp(args[0], "me", 2) == 0)
		usage(C_MEM);
	else if (strncmp(args[0], "mu", 2) == 0)
		usage(C_MUTE);
	else if (strncmp(args[0], "sc", 2) == 0)
		usage(C_SCAN);
	else if (strncmp(args[0], "sr", 2) == 0)
		usage(C_SCAN_REPORT);
	else if (strncmp(args[0], "se", 2) == 0)
		usage(C_SEARCH);
	else if (strncmp(args[0], "st", 2) == 0)
		usage(C_STATUS);
	else if (strncmp(args[0], "t", 1) == 0)
		usage(C_TALKGROUP);
	else if (strncmp(args[0], "v", 1) == 0)
		usage(C_VERSION);
	else
		usage(C_NONE);
}
