/* vi:set wm=0 ai tabstop=4 shiftwidth=4: */

/*
 * sctl - Bearcat scanner control program
 * Copyright (C) 2001 Ian Johnston <reaper@muppetlabs.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "utils.h"

/***********************************************************************
 * converts a frequency from the scanner into a human readable format
 ***********************************************************************/
double bc2human_freq(char *freq_string)
{
	int temp_freq;
	double real_freq;

	temp_freq = atoi(freq_string);
	real_freq = (double)temp_freq / 10000;

	return(real_freq);
}


/***********************************************************************
 * converts a frequency into the format understood by the scanner
 ***********************************************************************/
int human2bc_freq(char *freq_string)
{
	double temp_freq1;
	int temp_freq2;

	temp_freq1 = atof(freq_string);
	temp_freq2 = (int)(temp_freq1 * 10000);
	snprintf(freq_string, 9, "%08d", temp_freq2);

	return(1);
}


/***********************************************************************
 * Returns "on" or "off" depending on the status of the char sent to it.
 ***********************************************************************/
int bc2human_onoff(char status, char *result)
{
	if (status == 'F')
	{
		snprintf(result, 4, "OFF");
		return(1);
	}
	else if (status == 'N')
	{
		snprintf(result, 3, "ON");
		return(1);
	}

	return(-1);
}


/***********************************************************************
 * Check for error in scanner's reply
 ***********************************************************************/
void check_err(char *command, char *answer)
{
	char errorstr[512];

	if (strncmp("ERR", answer, 3) == 0)
	{
		snprintf(errorstr, 512,
		"The command failed because an error condition was returned.\n"
		"Command: %s\n"
		"Reply: %s\n"
		"This error usually occurs when a command unsupported by your scanner is\n"
		"executed, or a command is executed improperly.", command, answer);
		complain(errorstr);
		exit(EXIT_FAILURE);
	}
	else if (strncmp("NG", answer, 2) == 0)
	{
		snprintf(errorstr, 512,
		"The command failed because a \"No Good\" condition was returned.\n"
		"Command: %s\n"
		"Reply: %s\n"
		"This error usually occurs when you execute a command while in the wrong mode.\n"
		"For example, if you execute a command while in channel scan mode, but the\n"
		"command only works in manual mode, you will receive this error.", command, answer);
		complain(errorstr);
		exit(EXIT_FAILURE);
	}
}


/***********************************************************************
 * Unexpected reply
 ***********************************************************************/
void u_err(char *command, char *answer)
{
	char errorstr[512];

	snprintf(errorstr, 512,
	"An unexpected reply was returned.\n"
	"Command: %s\n"
	"Reply: %s\n"
	"Please file a bug report.", command, answer);
	complain(errorstr);
	exit(EXIT_FAILURE);
}


/***********************************************************************
 * print out an error message
 ***********************************************************************/
void complain(char *message)
{
	fprintf(stderr, "*** Error: %s\n", message);
}


/***********************************************************************
 * get the first channel in a bank
 ***********************************************************************/
int get_first_channel(char bank)
{
	switch (bank)
	{
		case 'A':
			return(1);
		case 'B':
			return(31);
		case 'C':
			return(61);
		case 'D':
			return(91);
		case 'E':
			return(121);
		case 'F':
			return(151);
		case 'G':
			return(181);
		case 'H':
			return(211);
		case 'I':
			return(241);
		case 'J':
			return(271);
		default:
			return(-1);
	}

	return(1);
}


/***********************************************************************
 * Check that frequencies entered by user are within range of scanner.
 ***********************************************************************/
int withinlimits(double freq)
{
	double maxfreq;
	double minfreq;
	char errorstr[100];

	if (scannermodel == BC245XLT || scannermodel == BC895XLT)
	{
		maxfreq = 956;
		minfreq = 29;
	}
	else if (scannermodel == BC780XLT)
	{
		maxfreq = 1300;
		minfreq = 26;  /* woohoo CB! */
	}
	else if (scannermodel == PRO2052)
	{
		maxfreq = 1300;
		minfreq = 29;
	}

	/* check for values exceeding minimum and maximum possible */
	if (freq > maxfreq)
	{
		snprintf(errorstr, 100, "Your scanner doesn't support frequencies above %.4f MHz.", maxfreq);
		complain(errorstr);
		return(-1);
	}
	else if (freq < minfreq)
	{
		snprintf(errorstr, 100, "Your scanner doesn't support frequencies below %.4f MHz.", minfreq);
		complain(errorstr);
		return(-1);
	}

	/* check for the various frequency holes */
	if ( ((freq > 54 && freq < 108) || (freq > 174 && freq < 406) || (freq > 512 && freq < 806)) &&
			scannermodel == BC245XLT )
	{
		snprintf(errorstr, 100, "The frequency you specified is not available on your scanner (%.4f MHz).", freq);
		complain(errorstr);
		return(-1);
	}
	else if ( ((freq > 54 && freq < 108) || (freq > 174 && freq < 216) || (freq > 512 && freq < 806)) &&
			scannermodel == BC895XLT )
	{
		snprintf(errorstr, 100, "The frequency you specified is not available on your scanner (%.4f MHz).", freq);
		complain(errorstr);
		return(-1);
	}
	else if ( ((freq > 512 && freq < 806) || (freq > 956 && freq < 1240)) &&
			scannermodel == BC780XLT )
	{
		snprintf(errorstr, 100, "The frequency you specified is not available on your scanner (%.4f MHz).", freq);
		complain(errorstr);
		return(-1);
	}
	else if ( ((freq > 54 && freq < 108) || (freq > 174 && freq < 179.75) || (freq > 512 && freq < 806) ||
			(freq > 956 && freq < 1240)) && scannermodel == PRO2052 )
	{
		snprintf(errorstr, 100, "The frequency you specified is not available on your scanner (%.4f MHz).", freq);
		complain(errorstr);
		return(-1);
	}

	/* inform user about uniden's cellular block */
	/* (values based on the testing of my 245) */
	if (freq > 869.98 && freq < 894.02)
	{
		snprintf(errorstr, 100, "This frequency is blocked on your scanner (%.4f MHz).", freq);
		complain(errorstr);
		return(-1);
	}

	return(1);
}


/***********************************************************************
 * prints out an S-meter scale, for use in status reporting commands
 * don't know if this will ever be applicable to other scanners, but it
 * will be easy to adapt.  macros are defined in config.h.
 ***********************************************************************/
void print_s_meter(char * slchar)
{
	int slevel, i;
	double step, units;

	if (scannermodel == BC895XLT)
	{
		step = ((float)S_895_MAX - (float)S_895_MIN) / (float)MAX_S_SCALE;
		slevel = atoi(slchar);
		units = ((float)slevel - (float)S_895_MIN) / step;
	}
	else /* add other scanner models here */
		return;

	putchar('|');
	for(i=1; i<MAX_S_SCALE; i++)
	{
		if (i < units)
			putchar('*');
		else
			putchar('-');
	}
	putchar('|');
}
